@@ -1,8 +1,9 @@
 /* -*- Mode: C; c-basic-offset: 8 -*- */
 /* libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998, 1999  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998, 1999, 2000  James Henstridge <james@daa.com.au>
  * Copyright (C) 1999 Miguel de Icaza (miguel@gnu.org)
  *
+ * glade-gnome.c: support for gnome widgets in libglade.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -237,6 +238,10 @@ dock_build_children(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 		   const char *longname)
 {
 	GList *tmp;
+	GnomeApp *app = NULL;
+
+	if (w->parent && w->parent && GNOME_IS_APP(w->parent->parent))
+		app = GNOME_APP(w->parent->parent);
 
 	for (tmp = info->children; tmp; tmp = tmp->next) {
 		GladeWidgetInfo *cinfo = tmp->data;
@@ -276,9 +281,14 @@ dock_build_children(GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 				else if (!strcmp(attr->name, "offset"))
 					offset = strtoul(attr->value, NULL, 0);
 			}
-			gnome_dock_add_item(GNOME_DOCK(w),
-					    GNOME_DOCK_ITEM(child), placement,
-					    band, position, offset, FALSE);
+			if (app)
+				gnome_app_add_dock_item(app,
+					GNOME_DOCK_ITEM(child), placement,
+					band, position, offset);
+			else
+				gnome_dock_add_item(GNOME_DOCK(w),
+					GNOME_DOCK_ITEM(child), placement,
+					band, position, offset, FALSE);
 		}
 	}
 }
@@ -855,6 +865,7 @@ static const gnomeuiinfo_map_t gnome_uiinfo_mapping[] = {
 	{ "HINT_ITEM", GNOMEUIINFO_MENU_HINT_ITEM(NULL, NULL) },
 	{ "NEW_GAME_ITEM", GNOMEUIINFO_MENU_NEW_GAME_ITEM(NULL, NULL) },
 	{ "NEW_ITEM", GNOMEUIINFO_MENU_NEW_ITEM(NULL, NULL, NULL, NULL) },
+	{ "NEW_SUBTREE", GNOMEUIINFO_MENU_NEW_SUBTREE(tmptree) },
 	{ "NEW_WINDOW_ITEM", GNOMEUIINFO_MENU_NEW_WINDOW_ITEM(NULL, NULL) },
 	{ "OPEN_ITEM", GNOMEUIINFO_MENU_OPEN_ITEM(NULL, NULL) },
 	{ "PASTE_ITEM", GNOMEUIINFO_MENU_PASTE_ITEM(NULL, NULL) },
@@ -1551,7 +1562,7 @@ pixmapmenuitem_new(GladeXML *xml, GladeWidgetInfo *info)
 	GtkWidget *wid;
 	GList *tmp;
 	char *label = NULL;
-	const char *stock_icon = NULL;
+	const char *stock_icon = NULL, *user_icon = NULL;
 	gboolean right = FALSE;
 
 	for (tmp = info->attributes; tmp; tmp = tmp->next) {
@@ -1563,6 +1574,8 @@ pixmapmenuitem_new(GladeXML *xml, GladeWidgetInfo *info)
 			stock_icon = get_stock_name(attr->value);
 		else if (!strcmp(attr->name, "right_justify"))
 			right = attr->value[0] == 'T';
+		else if (!strcmp(attr->name, "icon"))
+			user_icon = attr->value;
 	}
 	wid = gtk_pixmap_menu_item_new();
 	if (label) {
@@ -1592,6 +1605,13 @@ pixmapmenuitem_new(GladeXML *xml, GladeWidgetInfo *info)
 	}
 	if (stock_icon) {
 		GtkWidget *iconw = gnome_stock_new_with_icon(stock_icon);
+		gtk_pixmap_menu_item_set_pixmap(GTK_PIXMAP_MENU_ITEM(wid),
+						iconw);
+		gtk_widget_show(iconw);
+	} else if (user_icon) {
+		gchar *iconfile = glade_xml_relative_file(xml, user_icon);
+		GtkWidget *iconw = gnome_pixmap_new_from_file(iconfile);
+
 		gtk_pixmap_menu_item_set_pixmap(GTK_PIXMAP_MENU_ITEM(wid),
 						iconw);
 		gtk_widget_show(iconw);
