@@ -1,6 +1,8 @@
 /* -*- Mode: C; c-basic-offset: 8 -*- */
 /* libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998, 1999  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998, 1999, 2000  James Henstridge <james@daa.com.au>
+ *
+ * glade-gtk.c: support for GTK+ widgets in libglade
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -280,25 +282,47 @@ static void
 paned_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 		      const char *longname)
 {
-	GList *tmp;
+	GList *tmp, *tmp2;
 	GtkWidget *child;
+	gboolean resize, shrink;
 
 	tmp = info->children;
 	if (!tmp)
 		return;
 	
 	child = glade_xml_build_widget (xml, tmp->data, longname);
-	gtk_paned_add1 (GTK_PANED(w), child);
+	resize = FALSE; shrink = TRUE;
+	for (tmp2 = ((GladeWidgetInfo *)tmp->data)->child_attributes;
+	     tmp2; tmp2 = tmp2->next) {
+		GladeAttribute *attr = tmp2->data;
+
+		if (!strcmp(attr->name, "resize"))
+			resize = attr->value[0] == 'T';
+		else if (!strcmp(attr->name, "shrink"))
+			shrink = attr->value[0] == 'T';
+	}
+	gtk_paned_pack1 (GTK_PANED(w), child, resize, shrink);
 
 	tmp = tmp->next;
 	if (!tmp)
 		return;
 	
 	child = glade_xml_build_widget (xml, tmp->data, longname);
-	gtk_paned_add2 (GTK_PANED(w), child);
+	resize = TRUE; shrink = TRUE;
+	for (tmp2 = ((GladeWidgetInfo *)tmp->data)->child_attributes;
+	     tmp2; tmp2 = tmp2->next) {
+		GladeAttribute *attr = tmp2->data;
+
+		if (!strcmp(attr->name, "resize"))
+			resize = attr->value[0] == 'T';
+		else if (!strcmp(attr->name, "shrink"))
+			shrink = attr->value[0] == 'T';
+	}
+	gtk_paned_pack2 (GTK_PANED(w), child, resize, shrink);
 }
 
-static void note_change_page(GtkWidget *child, GtkNotebook *notebook)
+static void
+note_change_page(GtkWidget *child, GtkNotebook *notebook)
 {
 	gint page = gtk_notebook_page_num(notebook, child);
 
@@ -1588,7 +1612,45 @@ toolbar_new(GladeXML *xml, GladeWidgetInfo *info)
 static GtkWidget *
 progressbar_new(GladeXML *xml, GladeWidgetInfo *info)
 {
-	return gtk_progress_bar_new();
+	GtkWidget *ret = gtk_progress_bar_new();
+	GList *tmp;
+	gfloat value = 0, lower = 0, upper = 100;
+	gfloat xalign = 0.5, yalign = 0.5;
+
+	for (tmp = info->attributes; tmp; tmp = tmp->next) {
+		GladeAttribute *attr = tmp->data;
+
+		if (!strcmp(attr->name, "value"))
+			value = g_strtod(attr->value, NULL);
+		else if (!strcmp(attr->name, "lower"))
+			lower = g_strtod(attr->value, NULL);
+		else if (!strcmp(attr->name, "upper"))
+			upper = g_strtod(attr->value, NULL);
+		else if (!strcmp(attr->name, "activity_mode"))
+			gtk_progress_set_activity_mode(GTK_PROGRESS(ret),
+						       attr->value[0]=='T');
+		else if (!strcmp(attr->name, "bar_style"))
+			gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR(ret),
+			    glade_enum_from_string(GTK_TYPE_PROGRESS_BAR_STYLE,
+						   attr->value));
+		else if (!strcmp(attr->name, "orientation"))
+			gtk_progress_bar_set_orientation(GTK_PROGRESS_BAR(ret),
+			    glade_enum_from_string(
+			      GTK_TYPE_PROGRESS_BAR_ORIENTATION, attr->value));
+		else if (!strcmp(attr->name, "show_text"))
+			gtk_progress_set_show_text(GTK_PROGRESS(ret),
+						   attr->value[0] == 'T');
+		else if (!strcmp(attr->name, "text_xalign"))
+			xalign = g_strtod(attr->value, NULL);
+		else if (!strcmp(attr->name, "text_yalign"))
+			yalign = g_strtod(attr->value, NULL);
+		else if (!strcmp(attr->name, "format"))
+			gtk_progress_set_format_string(GTK_PROGRESS(ret),
+						       attr->value);
+	}
+	gtk_progress_configure(GTK_PROGRESS(ret), value, lower, upper);
+	gtk_progress_set_text_alignment(GTK_PROGRESS(ret), xalign, yalign);
+	return ret;
 }
 
 static GtkWidget *
@@ -2124,10 +2186,13 @@ hpaned_new(GladeXML *xml, GladeWidgetInfo *info)
 
 		if (!strcmp(attr->name, "handle_size"))
 			gtk_paned_set_handle_size(GTK_PANED(paned),
-						  g_strtod(attr->value, NULL));
+						  strtoul(attr->value,NULL,0));
 		else if (!strcmp(attr->name, "gutter_size"))
 			gtk_paned_set_gutter_size(GTK_PANED(paned),
-						  g_strtod(attr->value, NULL));
+						  strtoul(attr->value,NULL,0));
+		else if (!strcmp(attr, "position"))
+			gtk_paned_set_position(GTK_PANED(paned),
+					       strtol(attr->value, NULL, 0));
 	}
 	return paned;
 }
@@ -2143,10 +2208,13 @@ vpaned_new(GladeXML *xml, GladeWidgetInfo *info)
 
 		if (!strcmp(attr->name, "handle_size"))
 			gtk_paned_set_handle_size(GTK_PANED(paned),
-						  g_strtod(attr->value, NULL));
+						  strtoul(attr->value,NULL,0));
 		else if (!strcmp(attr->name, "gutter_size"))
 			gtk_paned_set_gutter_size(GTK_PANED(paned),
-						  g_strtod(attr->value, NULL));
+						  strtoul(attr->value,NULL,0));
+		else if (!strcmp(attr, "position"))
+			gtk_paned_set_position(GTK_PANED(paned),
+					       strtol(attr->value, NULL, 0));
 	}
 	return paned;
 }
