@@ -1,6 +1,8 @@
-//* -*- Mode: C; c-basic-offset: 8 -*- */
+/* -*- Mode: C; c-basic-offset: 8 -*- */
 /* libglade - a library for building interfaces from XML files at runtime
- * Copyright (C) 1998, 1999  James Henstridge <james@daa.com.au>
+ * Copyright (C) 1998, 1999, 2000  James Henstridge <james@daa.com.au>
+ *
+ * glade-xml.c: implementation of core public interface functions
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -50,6 +52,8 @@ static void glade_xml_destroy(GtkObject *object);
 static void glade_xml_build_interface(GladeXML *xml, GladeWidgetTree *tree,
 				      const char *root);
 
+GladeExtendedFunc *glade_xml_build_extended_widget = NULL;
+
 /**
  * glade_xml_get_type:
  *
@@ -456,6 +460,66 @@ glade_xml_signal_autoconnect_full (GladeXML *self, GladeXMLConnectFunc func,
 			     (GHFunc)autoconnect_full_foreach, &conn);
 }
 
+/**
+ * glade_xml_signal_connect_data:
+ * @self: the GladeXML object
+ * @handlername: the signal handler name
+ * @func: the signal handler function
+ * @user_data: the signal handler data
+ *
+ * In the glade interface descriptions, signal handlers are specified for
+ * widgets by name.  This function allows you to connect a C function to
+ * all signals in the GladeXML file with the given signal handler name.
+ *
+ * It differs from glade_xml_signal_connect since it allows you to
+ * specify the data parameter for the signal handler.  It is also a small
+ * demonstration of how to use glade_xml_signal_connect_full.
+ */
+typedef struct {
+	GtkSignalFunc func;
+	gpointer user_data;
+} connect_data_data;
+
+static void
+connect_data_connect_func(const gchar *handler_name, GtkObject *object,
+			  const gchar *signal_name, const gchar *signal_data,
+			  GtkObject *connect_object, gboolean after,
+			  gpointer user_data)
+{
+	connect_data_data *data = (connect_data_data *)user_data;
+
+	if (connect_object) {
+		if (after)
+			gtk_signal_connect_object_after(object, signal_name,
+							data->func,
+							connect_object);
+		else
+			gtk_signal_connect_object(object, signal_name,
+						  data->func,
+						  connect_object);
+	} else {
+		if (after)
+			gtk_signal_connect_after(object, signal_name,
+						 data->func, data->user_data);
+		else
+			gtk_signal_connect(object, signal_name,
+					   data->func, data->user_data);
+	}
+}
+
+void
+glade_xml_signal_connect_data (GladeXML *self, const char *handlername,
+			       GtkSignalFunc func, gpointer user_data)
+{
+	connect_data_data data;
+
+	data.func = func;
+	data.user_data = user_data;
+
+	glade_xml_signal_connect_full(self, handlername,
+				      connect_data_connect_func, &data);
+}
+
 /**
  * glade_xml_get_widget:
  * @self: the GladeXML object.
@@ -476,6 +540,47 @@ glade_xml_get_widget (GladeXML *self, const char *name)
 	return g_hash_table_lookup(self->priv->name_hash, name);
 }
 
+
+/**
+ * glade_xml_get_widget_prefix:
+ * @self: the GladeXML object.
+ * @name: the name of the widget.
+ *
+ * This function is used to get a list of pointers to the GtkWidget(s)
+ * with names that start with the string @name in the interface description.
+ * You would use this if you have to do something  to all of these widgets
+ * after loading.
+ *
+ * Returns: A list of the widget that match @name as the start of their
+ * name, or %NULL if none exists.
+ */
+typedef struct {
+	const gchar *name;
+	GList *list;
+} widget_prefix_data;
+
+static void
+widget_prefix_add_to_list (gchar *name, gpointer value,
+			   widget_prefix_data *data)
+{
+	if (!strncmp (data->name, name, strlen (data->name)))
+		data->list = g_list_prepend (data->list, value);
+}
+
+GList *
+glade_xml_get_widget_prefix (GladeXML *self, const gchar *prefix)
+{
+	widget_prefix_data data;
+
+	data.name = prefix;
+	data.list = NULL;
+
+	g_hash_table_foreach (self->priv->name_hash,
+			      (GHFunc) widget_prefix_add_to_list, &data);
+
+	return data.list;
+}
+
 /**
  * glade_xml_get_widget_by_long_name:
  * @self: the GladeXML object.
@@ -999,11 +1104,24 @@ glade_xml_build_widget(GladeXML *self, GladeWidgetInfo *info,
 	}
 	data = g_hash_table_lookup(widget_table, info->class);
 	if (data == NULL) {
-		char buf[50];
-		g_warning("unknown widget class '%s'", info->class);
-		g_snprintf(buf, 49, "[a %s]", info->class);
-		ret = gtk_label_new(buf);
-		gtk_widget_show(ret);
+		if (glade_xml_build_extended_widget) {
+			char *err = NULL;
+
+			ret = glade_xml_build_extended_widget(self, info, &err);
+			if (!ret) {
+				g_warning("%s", err);
+				ret = gtk_label_new(err);
+				g_free(err);
+				gtk_widget_show(ret);
+			}
+		} else {
+			char buf[50];
+			g_warning("unknown widget class '%s'", info->class);
+			g_snprintf(buf, 49, "[a %s]", info->class);
+			ret = gtk_label_new(buf);
+			gtk_widget_show(ret);
+		}
+		
 		return ret;
 	}
 	g_assert(data->new);
@@ -1087,7 +1205,27 @@ glade_xml_set_common_params(GladeXML *self, GtkWidget *widget,
 		GladeAttribute *attr = tmp->data;
 
 		if (!strcmp(attr->name, "events")) {
-			long events = strtol(attr->value, NULL, 0);
+			char *tmpptr, *endptr;
+			long events = strtol(attr->value, &endptr, 0);
+
+			/* format conversion error */
+			if (attr->value == endptr) {
+				events = 0;
+				tmpptr = attr->value;
+				while ((endptr = strchr(tmpptr, ' '))) {
+					char *str = g_strndup(tmpptr,
+							      endptr-tmpptr);
+					events |= glade_enum_from_string(
+						GTK_TYPE_GDK_EVENT_MASK, str);
+					g_free(str);
+					tmpptr = endptr;
+					while (tmpptr[0] == ' ' ||
+					       tmpptr[0] == '|')
+						tmpptr++;
+				}
+				events |= glade_enum_from_string(
+					GTK_TYPE_GDK_EVENT_MASK, tmpptr);
+			}
 			gtk_widget_set_events(widget, events);
 		} else if (!strcmp(attr->name, "extension_events")) {
 			GdkExtensionMode ex =
